from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
)

from pydantic_settings_sops import SOPSConfigSettingsSource


def test_get_settings():
    settings = TestSettings()
    assert settings.foobar == "foo"
    assert settings.foobar2 == "foo"

    settings2 = TestSettings(foobar="bar")
    assert settings2.foobar == "bar"
    assert settings2.foobar2 == "foo"


class TestSettings(BaseSettings):
    # todo: add json tests: https://github.com/nikaro/sopsy/issues/73
    model_config = SettingsConfigDict(
        yaml_file=["tests/resources/secrets.yaml", "tests/resources/secrets2.yaml"]
    )

    foobar: str
    foobar2: str

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: BaseSettings,  # type: ignore[override]
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> tuple[PydanticBaseSettingsSource, ...]:
        return (init_settings, SOPSConfigSettingsSource(settings_cls))



import importlib.metadata
import warnings

try:
    __version__ = importlib.metadata.version(__name__)
except importlib.metadata.PackageNotFoundError as e:  # pragma: no cover
    warnings.warn(f"Could not determine version of {__name__}\n{e!s}", stacklevel=2)
    __version__ = "unknown"

from pathlib import Path
from typing import Any

from pydantic_settings import BaseSettings
from pydantic_settings.sources import (
    DEFAULT_PATH,
    ConfigFileSourceMixin,
    InitSettingsSource,
    PathType,
)
from sopsy import Sops


class SOPSConfigSettingsSource(InitSettingsSource, ConfigFileSourceMixin):
    """
    A simple settings source class that loads variables via SOPS.
    """

    def __init__(
        self,
        settings_cls: BaseSettings,
        json_file: PathType | None = DEFAULT_PATH,
        yaml_file: PathType | None = DEFAULT_PATH,
    ):
        # currently, json is not working: https://github.com/nikaro/sopsy/issues/73
        self.json_file_path = (
            json_file
            if json_file != DEFAULT_PATH
            else settings_cls.model_config.get("json_file")
        )
        self.yaml_file_path = (
            yaml_file
            if yaml_file != DEFAULT_PATH
            else settings_cls.model_config.get("yaml_file")
        )
        self.data = self._read_files(self.json_file_path)
        self.data |= self._read_files(self.yaml_file_path)
        super().__init__(settings_cls, self.data)  # type: ignore[arg-type]

    def _read_file(self, file_path: Path) -> dict[str, Any]:
        sops = Sops(file_path)
        return sops.decrypt(to_dict=True)  # type: ignore[return-value]
